package MT::Plugin::LogViewer4MTOS;
use strict;
use MT;
use MT::Plugin;
use File::Spec;
use LogViewer4MTOS::LogViewer;
use LogViewer4MTOS::LogViewerUrls;
use LogViewer4MTOS::LogSummary;
use LogViewer4MTOS::LogReportSession;
use LogViewer4MTOS::LogViewerLastUrls;
use LogViewer4MTOS::LogViewerUserInfo;


use MT::Util qw( offset_time_list format_ts encode_url decode_url start_end_month wday_from_ts );

our $VERSION = '0.2';
our $SCHEMA_VERSION = '0.0612';

use base qw( MT::Plugin );

###################################### Init Plugin #####################################

@MT::Plugin::LogViewer4MTOS::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::LogViewer4MTOS({
    id => 'LogViewer4MTOS',
    key => 'logviewer4mtos',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    name => 'LogViewer4MTOS',
    author_name => 'Alfasado, Inc.',
    author_link => 'http://alfasado.net/',
    'version' => $VERSION,
    'schema_version' => $SCHEMA_VERSION,
    l10n_class => 'LogViewer4MTOS::L10N',
    settings => new MT::PluginSettings([
        ['index', { Default => 'index.html' }],
        ['trackurl', { Default => 'tracking.html' }],
        ['trackscript', { Default => 'tracking.js' }],
    ]),
    blog_config_template => 'log_viewew_config.tmpl',
});

sub load_tmpl {
    my $plugin = shift;
    my $tmpl = $plugin->SUPER::load_tmpl(@_);
    $tmpl->text($plugin->translate_templatized($tmpl->text));
    $tmpl;
}

MT->add_plugin($plugin);

our $Log = File::Spec->catdir($plugin->path, 'log');

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        object_types => {
            'mtoslogviewer' =>  'LogViewer4MTOS::LogViewer',
            'mtoslogviewerurls' =>  'LogViewer4MTOS::LogViewerUrls',
            'mtoslogsummary' =>  'LogViewer4MTOS::LogSummary',
            'mtoslogviewerlasturls' =>  'LogViewer4MTOS::LogViewerLastUrls',
            'mtoslogreportsession' =>  'LogViewer4MTOS::LogReportSession',
            'mtoslogvieweruserinfo' =>  'LogViewer4MTOS::LogViewerUserInfo',
        },
        applications => {
            cms => {
                menus => {
                    'tools:import_log' => {
                        label => $plugin->translate('Import Log'),
                        mode => 'import_log',
                        order => 11000,
                        permission => 'view_blog_log',
                    },
                    'tools:view_access_log' => {
                        label => $plugin->translate('View Report'),
                        mode => 'view_access_log',
                        order => 10000,
                        permission => 'view_blog_log',
                    },
                },
                methods => {
                    import_log =>
                        'MT::Plugin::LogViewer4MTOS::_import_log',
                    make_report =>
                        'MT::Plugin::LogViewer4MTOS::_make_report',
                    last_report =>
                        'MT::Plugin::LogViewer4MTOS::_last_report',
                    view_access_log =>
                        'MT::Plugin::LogViewer4MTOS::_view_access_log',
                    view_by_daily =>
                        'MT::Plugin::LogViewer4MTOS::_view_by_daily',
                },
            },
        },
        tags => {
            block => {
                'LVHotEntries' => \&_hot_entries,
            },
            function => {
                'LVTrackingLog' => \&_tracking_log,
                'LVTrackBasename' => \&_trackurl,
                'LVTrackScript' => \&_trackscript,
                'LVDirectoryIndexFileName' => \&_index,
                'LVTrackingSearchLog' => \&_tracking_log,
            },
        }
   });
}

########################################## Tags ########################################

sub _hot_entries {
    my ($ctx, $args, $cond) = @_;
    my $tokens = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');
    my $blog = $ctx->stash('blog');
    my $blog_id = $blog->id;
    my $lastn = $args->{lastn};
    my $multiblog = $args->{multiblog};
    my $period = $args->{period};
    my $class = $args->{class};
    unless ($class) {
        $class = 'entry';
    }
    if ( ($period eq 'last1day') || ($period eq 'last1days') ) {
        $period = 'last1days';
    } elsif  ($period eq 'last7day')  {
        $period = 'last7days';
    } else {
        $period = 'last7days';
    }
    my $sort_order = $args->{sort_order};
    unless ($sort_order) {
        $sort_order = 'descend';
    }
    my $terms = { 
                  class   => $class,
                  status  => MT::Entry::RELEASE(),
                 };
    my $params = {
                  'join' => ['LogViewer4MTOS::LogViewerLastUrls', 'entry_id',
                 { object_type => 'entry',
                   period => $period,
                 },
                 {
                   sort => 'counter',
                   direction => $sort_order,
                   unique => 1,
                 } ] };                             
    if ($lastn) {
        $params->{limit} = $lastn;
    }
    unless ($multiblog) {
        $terms->{blog_id}  = $blog_id;
        $params->{blog_id} = $blog_id;
    }
    my @items = MT::Entry->load( $terms , $params );
    my $res = '';
    foreach my $entry (@items) {
        local $ctx->{__stash}{'entry'} = $entry;
        my $out = $builder->build($ctx, $tokens, $cond);
        $res .= $out;
    }
    $res;
}

sub _trackurl {
	my ($ctx, $args, $cond) = @_;
	my $blog = $ctx->stash('blog');
	my $config_id = 'blog:'.$blog->id;
	return $plugin->get_config_value('trackurl', $config_id);
}

sub _trackscript {
	my ($ctx, $args, $cond) = @_;
	my $blog = $ctx->stash('blog');
	my $config_id = 'blog:'.$blog->id;
	return $plugin->get_config_value('trackscript', $config_id);
}

sub _index {
	my ($ctx, $args, $cond) = @_;
	my $blog = $ctx->stash('blog');
	my $config_id = 'blog:'.$blog->id;
	return $plugin->get_config_value('index', $config_id);
}

########################################## Main ########################################

sub _view_by_daily {
    my $app = shift;
    my $blog = $app->blog;
    my $blog_id;
    unless (defined $blog) {
        $blog = MT::Blog->load( undef, { limit => 1 } );
    } else {
        $blog_id = $blog->id;
    }
    my $q = $app->param;
    my @tl = &offset_time_list(time, $blog);
    my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
    my $period_key = $q->param('period_key');
    my $period = $q->param('period');
    unless ($period) {
        $period = 'monthly';
    }
    
    my @summaries; my $max_count; my $daily_total; my $daily_summary; my $t_blog_id;
    unless ($blog_id) {
        $blog_id = 0;
    }
    
    if ($period eq 'monthly') {
        unless ($period_key) {
            $period_key = substr($ts, 0, 6);
        }
        @summaries = LogViewer4MTOS::LogSummary->load( { blog_id => $blog_id,
                                                    period_month => $period_key,
                                                    period => 'daily' },
                                                  { sort   => 'period_key',
                                                  } );
        $max_count = LogViewer4MTOS::LogSummary->load( { blog_id => $blog_id,
                                                    period_month => $period_key,
                                                    period => 'daily' },
                                                 { sort   => 'counter',
                                                   direction => 'descend',
                                                   limit => 1,
                                                  } );
        $daily_summary = LogViewer4MTOS::LogSummary->load( { blog_id => $blog_id,
                                                    period_key => $period_key,
                                                    period => 'monthly' } );
    } elsif ($period eq 'yearly') {
        unless ($period_key) {
            $period_key = substr($ts, 0, 4);
        }

        @summaries = LogViewer4MTOS::LogSummary->load( { blog_id => $blog_id,
                                                    period_year => $period_key,
                                                    period => 'monthly' },
                                                  { sort   => 'period_key',
                                                  } );

        $max_count = LogViewer4MTOS::LogSummary->load( { blog_id => $blog_id,
                                                    period_year => $period_key,
                                                    period => 'monthly' },
                                                 { sort   => 'counter',
                                                   direction => 'descend',
                                                   limit => 1,
                                                  } );
        $daily_summary = LogViewer4MTOS::LogSummary->load( { blog_id => $blog_id,
                                                    period_key => $period_key,
                                                    period => 'yearly' } );
    } elsif ($period eq 'all') {
        @summaries = LogViewer4MTOS::LogSummary->load( { blog_id => $blog_id,
                                                    period => 'yearly' },
                                                  { sort   => 'period_key',
                                                  } );

        $max_count = LogViewer4MTOS::LogSummary->load( { blog_id => $blog_id,
                                                    period => 'yearly' },
                                                 { sort   => 'counter',
                                                   direction => 'descend',
                                                   limit => 1,
                                                  } );
        $daily_summary = LogViewer4MTOS::LogSummary->load( { blog_id => $blog_id,
                                                    period => 'all' } );
    }
    my $year_key = substr($period_key, 0, 4);
    my $month_key = substr($period_key, 0, 6);
    my $meta_entry_total = 0; my $meta_page_total = 0; my $meta_category_total = 0;
    my $meta_daily_total = 0; my $meta_index_total = 0;
    my $meta_other_total = 0; my $meta_cout_user = 0;
    if (defined $daily_summary) {
        $daily_total = $daily_summary->counter + 0;
        $meta_entry_total = $daily_summary->counter_entry + 0;
        $meta_page_total = $daily_summary->counter_page + 0;
        $meta_category_total = $daily_summary->counter_category + 0;
        $meta_daily_total = $daily_summary->counter_daily + 0;
        $meta_index_total = $daily_summary->counter_index + 0;
        $meta_other_total = $daily_summary->counter_other + 0;
        $meta_cout_user = $daily_summary->counter_user + 0;
    }
    my $max;
    if (defined $max_count) {
        $max = $max_count->counter;
        $max = 200 / $max;
    }
    my $res;
    my %param; my $yy; my $mm;
    my @tmpl_loop; my $odd = 1;
    for my $summary (@summaries) {
        my $summary_date = $summary->period_key;
        my $script_period_key = $summary_date;
        my $yy = substr($summary_date, 0, 4);
        my $mm = substr($summary_date, 4, 2);
        my $dd = substr($summary_date, 6, 2);
        my $script_period;
        if ($period eq 'all') {
            $script_period = 'yearly';
        } else {
            if ($period eq 'monthly') {
                $script_period = 'daily';
                $summary_date = "$yy/$mm/$dd";
                my $wday = &wday_from_ts($yy, $mm, $dd);
                $summary_date .= ' (' . &_int2wday($wday) . ')';
            } elsif ($period eq 'yearly') {
                $script_period = 'monthly';
                $summary_date = "$yy/$mm";
            }
        }
        push @tmpl_loop,
            {
                date_label       => $summary_date,
                page_counter     => $summary->counter,
                script_period_key=> $script_period_key,
                script_period    => $script_period,
                page_percent     => (int (($summary->counter / $daily_total) * 1000)) / 10,
                page_bar_length  => int($max * $summary->counter) + 1,
                counter_entry    => $summary->counter_entry + 0,
                counter_page    => $summary->counter_page + 0,
                counter_category => $summary->counter_category + 0,
                counter_daily    => $summary->counter_daily + 0,
                counter_other    => $summary->counter_other + 0,
                counter_index    => $summary->counter_index + 0,
                counter_user     => $summary->counter_user + 0,
                odd => $odd,
            };
        if ($odd == 1) { $odd = 0 } else { $odd = 1 };
    }
    my $page_title;
    if ($period eq 'all') {
        $page_title = $plugin->translate('All Report');
    } else {
        if ($period eq 'yearly') {
            $page_title = $plugin->translate('Yearly Report');
        } elsif ($period eq 'monthly') {
            $page_title = $plugin->translate('Monthly Report');
        }
        $page_title .= ' ( ' . substr($period_key, 0, 4);
        if ($period eq 'monthly') {
            $page_title .= '/' . substr($period_key, 4, 2);
        }
        $page_title .= ' )';
    }

    $param{'page_title'} = $page_title;
    $param{'view_by_daily'} = 1;
    $param{'rank_label'} = $plugin->translate('Rank');
    $param{'table_header'} = $plugin->translate('Title');
    $param{'counter_label'} = $plugin->translate('Access');
    $param{'type_label'} = $plugin->translate('Type');
    $param{'last_access_label'} = $plugin->translate('Last Access');
    $param{'no_select_label'} = $plugin->translate('Select...');
    $param{'meta_total'} = $daily_total;
    $param{'meta_entry_total'} = $meta_entry_total;
    $param{'meta_page_total'} = $meta_page_total;
    $param{'meta_category_total'} = $meta_category_total;
    $param{'meta_daily_total'} = $meta_daily_total;
    $param{'meta_index_total'} = $meta_index_total;
    $param{'meta_other_total'} = $meta_other_total;
    $param{'meta_cout_user'} = $meta_cout_user;
    $param{'all_label'} = $plugin->translate('All');
    $param{'last7days_label'} = $plugin->translate('Last 7 days');
    $param{'last1days_label'} = $plugin->translate('Last 24 hours');
    $param{'label_total'} = $plugin->translate('Total');
    $param{'label_last'} = $plugin->translate('Last');
    $param{'label_days'} = $plugin->translate('Days');
    $param{'label_total'} = $plugin->translate('Total');
    $param{'pv_label'} = $plugin->translate('Page View');
    $param{'entry_th'} = $plugin->translate('Entry ');
    $param{'page_th'} = $plugin->translate('Page ');
    $param{'category_th'} = $plugin->translate('Cate<br />-gory');
    $param{'index_th'} = $plugin->translate('Index ');
    $param{'daily_th'} = $plugin->translate('Daily ');
    $param{'other_th'} = $plugin->translate('Other ');
    $param{'user_th'} = $plugin->translate('User ');
    $app->{plugin_template_path} = File::Spec->catdir($plugin->path,'tmpl');
    $param{'tmpl_loop'} = \@tmpl_loop;
    
    my @year_list = LogViewer4MTOS::LogSummary->load( { 
                                                 blog_id => $blog_id,
                                                 period     => 'yearly', },
                                               { sort       => 'period_key',
                                                 direction  => 'descend'} );

    my @month_list = LogViewer4MTOS::LogSummary->load( { 
                                                 blog_id => $blog_id,
                                                 period_year => $year_key,
                                                 period     => 'monthly', },
                                               { sort       => 'period_key',
                                                 direction  => 'descend'} );
                                                 
    my @opt_year_loop;
    for my $opt_year (@year_list) {
        my $ts = $opt_year->period_key;
        my $current_year = 0;
        if ( ($period ne 'monthly') && ($year_key == $ts) ) {
            if ($period eq 'yearly') {
                $current_year = 1;
            }
        }
        $ts .= '010101000000';
        push @opt_year_loop,
            {
                year_key     => $opt_year->period_key,
                year_label   => &format_ts('%Y', $ts, undef),
                current_year => $current_year,
            };
    }
    my @opt_month_loop; 
    for my $opt_month (@month_list) {
        my $ts = $opt_month->period_key;
        my $current_month = 0;
        if ($period eq 'monthly') {
            if ($ts eq $period_key) {
                $current_month = 1;
            }
        }
        $ts .= '01000000';
        push @opt_month_loop,
            {
                month_key     => $opt_month->period_key,
                month_label   => &format_ts('%Y/%m', $ts, undef),
                current_month => $current_month,
            };
    }
    
    my @day_list;
    if ($period eq 'monthly') {
        @day_list = LogViewer4MTOS::LogSummary->load( { blog_id      => $blog_id,
                                                   period_month => $month_key,
                                                   period       => 'daily', },
                                                 { sort         => 'period_key',
                                                   direction    => 'descend'} );
    }

    my @opt_day_loop; 
    for my $opt_day (@day_list) {
        my $ts = $opt_day->period_key;
        $ts .= '000000';
        push @opt_day_loop,
            {
                day_key     => $opt_day->period_key,
                day_label   => &format_ts('%Y/%m/%d', $ts, undef),
            };
    }

    $param{'opt_year_loop'} = \@opt_year_loop;
    $param{'opt_month_loop'} = \@opt_month_loop;
    $param{'opt_day_loop'} = \@opt_day_loop;
    my $tmpl = 'log_view.tmpl';
    return $app->build_page($tmpl, \%param);
}

sub _view_access_log {
    my $app = shift;
    my $blog = $app->blog;
    my $q = $app->param;
    my $period = $q->param('period');
    unless ($period) {
        $period = 'all';
    }
    my $offset = $q->param('offset');
    unless ($offset) {
        $offset = 0;
    }
    my $limit = $q->param('limit');
    unless ($limit) {
        $limit = 1000;
    }
    my $period_key = $q->param('period_key');
    my $blog_id;
    unless (defined $blog) {
        $blog = MT::Blog->load( undef, { limit => 1 } );
        $blog_id = 0;
    } else {
        $blog_id = $blog->id;
    }
    my $year_key; my $month_key;
    my $view_last;
    unless ($period_key) {
        my @tl = &offset_time_list(time, $blog);
        if ($period eq 'yearly') {
            $period_key = sprintf "%04d", $tl[5]+1900;
        } elsif ($period eq 'monthly') {
            $period_key = sprintf "%04d%02d", $tl[5]+1900, $tl[4]+1;
        } elsif ($period eq 'daily') {
            $period_key = sprintf "%04d%02d%02d", $tl[5]+1900, $tl[4]+1, $tl[3];
        }
        $month_key = sprintf "%04d%02d", $tl[5]+1900, $tl[4]+1;
        $year_key  = sprintf "%04d", $tl[5]+1900;
    } else {
        $month_key = substr($period_key, 0, 6);
        $year_key  = substr($period_key, 0, 4);
    }
    my @log_urls; my $max_urls; my $period_label;
    my $terms = { 'period' => $period };
    if ($blog_id) {
        $terms->{'blog_id'} = $blog_id;
    }
    my $total = 0; my $entry_total = 0; my $page_total = 0; my $category_total = 0;
    my $daily_total = 0; my $index_total = 0; my $other_total = 0; my $count_user = 0;
    my $log_obj;
    if ($period eq 'all') {
        $period_label = $plugin->translate("All");
        $log_obj = LogViewer4MTOS::LogSummary->load( { 'blog_id' => $blog_id, 'period' => 'all' } );
    } elsif ($period eq 'last7days') {
        $period_label = $plugin->translate("Last 7 days");
        $view_last = 7;
        $log_obj = LogViewer4MTOS::LogSummary->load( { 'blog_id' => $blog_id, 'period' => 'last7days' } );
    } elsif ($period eq 'last1days') {
        $view_last = 1;
        $period_label = $plugin->translate("Last 24 hours");
        $log_obj = LogViewer4MTOS::LogSummary->load( { 'blog_id' => $blog_id, 'period' => 'last1days' } );
    } elsif ( ($period eq 'yearly') || ($period eq 'monthly') || ($period eq 'daily') ) {
        $terms->{'period_key'} = $period_key;
        my $yyyy = substr($period_key, 0, 4);
        my $mm = substr($period_key, 4, 2);
        my $ts; my $start, my $end;
        if ($period eq 'yearly') {
            $period_label = $plugin->translate("Yearly") . ' : ' . $yyyy;
        } elsif ($period eq 'monthly') {
            $period_label = $plugin->translate("Monthly") . ' : ' . $yyyy . '/' . $mm;
        } elsif ($period eq 'daily') {
            my $dd = substr($period_key, 6, 2);
            $period_label = $plugin->translate("Daily") . ' : ' . $yyyy . '/' . $mm . '/' .$dd;
        }
        $log_obj = LogViewer4MTOS::LogSummary->load( { 'blog_id' => $blog_id,
                                                  'period' => $period,
                                                  'period_key' => $period_key,} );
    }
    if (defined $log_obj) {
        $total = $log_obj->counter + 0;
        $entry_total = $log_obj->counter_entry + 0;
        $page_total = $log_obj->counter_page + 0;
        $daily_total = $log_obj->counter_daily + 0;
        $category_total = $log_obj->counter_category + 0;
        $index_total = $log_obj->counter_index + 0;
        $other_total = $log_obj->counter_other + 0;
        $count_user = $log_obj->counter_user + 0;
    }

    if ( ($period eq 'last1days') || ($period eq 'last7days') ) {
        $max_urls = LogViewer4MTOS::LogViewerLastUrls->load( $terms,
                                                    { 'sort'   => 'counter',
                                                      'limit'  => 1,
                                                      'direction'  => 'descend' });

        @log_urls = LogViewer4MTOS::LogViewerLastUrls->load( $terms,
                                                    { 'sort'   => 'counter',
                                                      'offset' => $offset,
                                                      'limit'  => $limit,
                                                      'direction'  => 'descend' });
    } else {
        $max_urls = LogViewer4MTOS::LogViewerUrls->load( $terms,
                                                    { 'sort'   => 'counter',
                                                      'limit'  => 1,
                                                      'direction'  => 'descend' });
        @log_urls = LogViewer4MTOS::LogViewerUrls->load( $terms,
                                                    { 'sort'   => 'counter',
                                                      'offset' => $offset,
                                                      'limit'  => $limit,
                                                      'direction'  => 'descend' });
    }
    my $res;
    my %param;
    my @tmpl_loop; my $odd = 1;
    my $rank_start = $offset + 1;
    my $rank_end = $offset + $limit;
    $param{'rank_label'} = $plugin->translate('Rank');
    $param{'table_header'} = $plugin->translate('Title');
    $param{'counter_label'} = $plugin->translate('Access');
    $param{'type_label'} = $plugin->translate('Type');
    $param{'last_access_label'} = $plugin->translate('Last Access');
    $param{'no_select_label'} = $plugin->translate('Select...');
    $param{'meta_total'} = $total;
    $param{'meta_entry_total'} = $entry_total;
    $param{'meta_page_total'} = $page_total;
    $param{'meta_category_total'} = $category_total;
    $param{'meta_daily_total'} = $daily_total;
    $param{'meta_index_total'} = $index_total;
    $param{'meta_other_total'} = $other_total;
    $param{'meta_cout_user'} = $count_user;
    $param{'all_label'} = $plugin->translate('All');
    $param{'last7days_label'} = $plugin->translate('Last 7 days');
    $param{'last1days_label'} = $plugin->translate('Last 24 hours');
    $param{'label_total'} = $plugin->translate('Total');
    $param{'label_last'} = $plugin->translate('Last');
    $param{'label_days'} = $plugin->translate('Days');
    $param{'view_last'} = $view_last;
    $param{'view_access_log'} = 1;
    my $i = $offset;
    my $max;
    if (defined $max_urls) {
        $max = $max_urls->counter;
        $max = 60 / $max;
    }
    for my $item (@log_urls) {
        $i ++;
        my $last_access = $item->last_access;
        $last_access = &format_ts(undef, $last_access, undef);
        my $page_title = $item->title;
        $page_title =~ s/\\'/'/g;
        $page_title =~ s/&amp;/&/g;
        push @tmpl_loop,
            {
                page_rank    => $i,
                page_url     => $item->url,
                page_name    => $page_title,
                page_type    => $item->object_type,
                page_counter => $item->counter,
                page_percent => (int (($item->counter / $total) * 1000)) / 10,
                page_bar_length => int($max * $item->counter) + 1,
                last_access  => $last_access,
                odd => $odd,
            };
        if ($odd == 1) { $odd = 0 } else { $odd = 1 };
    }
    my @year_list; my @month_list;
    @year_list = LogViewer4MTOS::LogSummary->load( { 
                                                 blog_id => $blog_id,
                                                 period     => 'yearly', },
                                               { sort       => 'period_key',
                                                 direction  => 'descend'} );
    if ($period ne 'all') {
        @month_list = LogViewer4MTOS::LogSummary->load( { 
                                                     blog_id => $blog_id,
                                                     period_year => $year_key,
                                                     period     => 'monthly', },
                                                   { sort       => 'period_key',
                                                     direction  => 'descend'} );
    }
    my @opt_year_loop;
    for my $opt_year (@year_list) {
        my $ts = $opt_year->period_key;
        my $current_year = 0;
        if ( ($period ne 'all') && ($period ne 'last7days') && ($period ne 'last1days') && ($year_key == $ts) ) {
            if ($period eq 'yearly') {
                $current_year = 1;
            }
        }
        $ts .= '010101000000';
        push @opt_year_loop,
            {
                year_key     => $opt_year->period_key,
                year_label   => &format_ts('%Y', $ts, undef),
                current_year => $current_year,
            };
    }
    my @opt_month_loop; 
    for my $opt_month (@month_list) {
        my $ts = $opt_month->period_key;
        my $current_month = 0;
        if ($month_key == $ts) {
            if ($period eq 'monthly') {
                $current_month = 1;
            }
        }
        $ts .= '01000000';
        push @opt_month_loop,
            {
                month_key     => $opt_month->period_key,
                month_label   => &format_ts('%Y/%m', $ts, undef),
                current_month => $current_month,
            };
    }
    my @day_list;
    if ($period ne 'all') {
        @day_list = LogViewer4MTOS::LogSummary->load( { blog_id      => $blog_id,
                                                   period_month => $month_key,
                                                   period       => 'daily', },
                                                 { sort         => 'period_key',
                                                   direction    => 'descend'} );
    }
    my @opt_day_loop; 
    for my $opt_day (@day_list) {
        my $ts = $opt_day->period_key;
        my $current_day = 0;
        if ($period_key == $ts) {
            if ($period eq 'daily') {
                $current_day = 1;
            }
        }
        $ts .= '000000';
        push @opt_day_loop,
            {
                day_key     => $opt_day->period_key,
                day_label   => &format_ts('%Y/%m/%d', $ts, undef),
                current_day => $current_day,
            };
    }
    my $page_title = $plugin->translate('URL Report') . ' ( ' . $period_label . ' : ' . $plugin->translate('Rank');
    if ($i) {
        $page_title .= ' ' . $rank_start . ' - ' . $i;
    }
    $page_title .= ' )';
    $param{'page_title'} = $page_title;
    $app->{plugin_template_path} = File::Spec->catdir($plugin->path,'tmpl');
    $param{'tmpl_loop'} = \@tmpl_loop;
    $param{'opt_year_loop'} = \@opt_year_loop;
    $param{'opt_month_loop'} = \@opt_month_loop;
    $param{'opt_day_loop'} = \@opt_day_loop;
    my $tmpl = 'log_view.tmpl';
    return $app->build_page($tmpl, \%param);
}

sub _import_log {
    my $app = shift;
    my $blog = MT::Blog->load( undef, { limit => 1 } );
    my $saved_log; my $return_url;
    use Encode::Guess qw/ shiftjis euc-jp 7bit-jis /;
    use Encode qw/ decode /;

    my $index_file = $plugin->get_config_value('index');
    my $log = File::Spec->catfile($Log, 'access_log.log');
    unless (-f $log) {
        return $app->trans_error('File [_1] was not found.', $log);
    }
    my @tl = &offset_time_list(time, $blog);
    my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
    my $session_key = $app->make_magic_token;
    my $new_log = $log . '.' . $ts.$session_key . '.log';

    #my $new_log = $log;
    rename($log, $new_log);
    # `touch $log`;
    open(FH, $new_log);
    while(<FH>){
        my $line = $_;
        my @datas = split(/\t/, $line);
        my $timestamp = $datas[0];
        my $blog_id = $datas[1];
        my $object_type = $datas[2];
        my $object_id = $datas[3];
        my $url = $datas[4];
        my $title = &decode_url($datas[5]);
        $title =~ s/&amp;/&/g;
        my $enc = guess_encoding ( $title );
        if ( ref $enc ) { my $utf8 = decode ( $enc->name , $title ); }
        
        my $search_str = $datas[6];
        my $referrer = $datas[7];
        my $referrer_self = $datas[8];
        my $referrersite;
        unless ($referrer_self) {
            if ($referrer) {
                $referrer_self = 2;
                $referrersite = $referrer;
                $referrersite =~ s/(^https?:\/\/.*?)\/.*$/$1/;
                $referrersite .= '/';
            } else {
                $referrer_self = 0;
            }
        }
        my $user_name = $datas[9];
        my $user_id = $datas[10];
        my $remote_addr = $datas[11];
        my $ua = $datas[12];
        my $userinfo = $remote_addr . $ua . $user_id . $user_name;
        if (eval { require Digest::MD5; 1; }) {
            $userinfo = Digest::MD5::md5_hex($userinfo);
        } else {
            $userinfo = &encode_url($userinfo);
            $userinfo = substr($userinfo, 0, 255);
        }
        my $userinfo_record = LogViewer4MTOS::LogViewerUserInfo->load( { userinfo => $userinfo } );
        unless (defined $userinfo_record) {
            $userinfo_record = LogViewer4MTOS::LogViewerUserInfo->new;
            $userinfo_record->first_access($timestamp);
            $userinfo_record->userinfo($userinfo);
            $userinfo_record->remote_addr($remote_addr);
            $userinfo_record->user_name($user_name);
            $userinfo_record->user_id($user_id);
            $userinfo_record->save
                or return $app->trans_error('Error saving userinfo record: [_1]', $userinfo_record->errstr);
        }
        my $log_record = LogViewer4MTOS::LogViewer->new;
        $log_record->timestamp($timestamp);   
        $log_record->ts_year(substr($timestamp, 0, 4));
        $log_record->ts_month(substr($timestamp, 0, 6));
        $log_record->ts_day(substr($timestamp, 0, 8));
        $log_record->blog_id($blog_id);
        $log_record->object_type($object_type);
        $log_record->object_id($object_id);
        if ( ($object_type eq 'entry') || ($object_type eq 'page') ) {
            $log_record->entry_id($object_id);
        }
        $log_record->url($url);
        $log_record->title($title);
        $log_record->search_str($search_str);
        $log_record->referrer($referrer);
        $log_record->referrersite($referrersite);
        $log_record->referrer_self($referrer_self);
        $log_record->user_name($user_name);
        $log_record->user_id($user_id);
        $log_record->remote_addr($remote_addr);
        $log_record->import_time($ts);
        $log_record->session_key($session_key);
        $log_record->userinfo($userinfo);
        $log_record->userinfo_id($userinfo_record->id);
        $log_record->save
                or return $app->trans_error('Error saving log record: [_1]', $log_record->errstr);
        $saved_log = 1;
    }
    close(FH);
    
    # make_report, ts, session_key
    if ($saved_log) {
        my $sess = LogViewer4MTOS::LogReportSession->new;
        $sess->ts($ts);
        $sess->session_key($session_key);
        $sess->key('all');
        $sess->task('url');
        $sess->save 
                or return $app->trans_error('Error saving log record urls: [_1]', $sess->errstr);
                
        $sess = LogViewer4MTOS::LogReportSession->new;
        $sess->ts($ts);
        $sess->session_key($session_key);
        $sess->key('all');
        $sess->task('summary');
        $sess->save 
                or return $app->trans_error('Error saving log record urls: [_1]', $sess->errstr);

        $sess = LogViewer4MTOS::LogReportSession->new;
        $sess->ts($ts);
        $sess->session_key($session_key);
        $sess->key('yearly');
        $sess->task('url');
        $sess->save 
                or return $app->trans_error('Error saving log record urls: [_1]', $sess->errstr);
                
        $sess = LogViewer4MTOS::LogReportSession->new;
        $sess->ts($ts);
        $sess->session_key($session_key);
        $sess->key('yearly');
        $sess->task('summary');
        $sess->save 
                or return $app->trans_error('Error saving log record urls: [_1]', $sess->errstr);

        $sess = LogViewer4MTOS::LogReportSession->new;
        $sess->ts($ts);
        $sess->session_key($session_key);
        $sess->key('monthly');
        $sess->task('url');
        $sess->save 
                or return $app->trans_error('Error saving log record urls: [_1]', $sess->errstr);
                
        $sess = LogViewer4MTOS::LogReportSession->new;
        $sess->ts($ts);
        $sess->session_key($session_key);
        $sess->key('monthly');
        $sess->task('summary');
        $sess->save 
                or return $app->trans_error('Error saving log record urls: [_1]', $sess->errstr);
                
        $sess = LogViewer4MTOS::LogReportSession->new;
        $sess->ts($ts);
        $sess->session_key($session_key);
        $sess->key('daily');
        $sess->task('url');
        $sess->save 
                or return $app->trans_error('Error saving log record urls: [_1]', $sess->errstr);
                
        $sess = LogViewer4MTOS::LogReportSession->new;
        $sess->ts($ts);
        $sess->session_key($session_key);
        $sess->key('daily');
        $sess->task('summary');
        $sess->save 
                or return $app->trans_error('Error saving log record urls: [_1]', $sess->errstr);
                                                              
        $return_url = $app->uri(mode => 'make_report', args => {'ts' => $ts,
                                                                'session_key' => $session_key,
                                                                'key' => 'all', 'task' => 'url' } );
    }

    $app->{plugin_template_path} = File::Spec->catdir($plugin->path,'tmpl');
    my %param;
    $param{'page_title'} = $plugin->translate('Import Log');
    $param{'page_msg'} = $plugin->translate('No access log.');
    if ($saved_log) {
        my $base = $app->base;
        my $js = <<"JS";
var counter = 0;
var eid;
function func() {
    if (counter != 0) {
        clearTimeout(eid);
        counter = 0;
    } else {
        counter = 1;
        eid = setTimeout("func(window.location.href = '$base$return_url')", 200);
    }
}
func();
JS
        $param{'location_redirect'} = $js;
        $param{'page_title'} = $plugin->translate('Rebuilding Report');
        $param{'rebuild_report'} = 1;
        $param{'page_msg'} = $plugin->translate('Rebuilding all url report...');
        #$param{'location_redirect'} = 'var id = setTimeout(window.location.href = "' . $app->base . $return_url . '"), 1000);';
    }
    my $tmpl = 'import_log.tmpl';
    return $app->build_page($tmpl, \%param);
}

sub _last_report {
    my $app = shift;
    my $q = $app->param;
    my $blog = $app->blog;
    my $blog_id = $q->param('blog_id');
    my $object_type = $q->param('object_type');
    my $period = $q->param('period');
    my $offset = $q->param('offset');
    unless ($offset) {
        $offset = 0;
    }
    my $limit = $q->param('limit');
    unless ($limit) {
        $limit = 1000;
    }
    my $period_sec;
    my $total = 0; my $entry_total = 0; my $page_total = 0; my $category_total = 0;
    my $daily_total = 0; my $index_total = 0; my $other_total = 0;
    unless ($period) {
        $period_sec = 86400;
    } else {
        #$period =~ s/last([0-9]{1,})days/$1/i;
        $period_sec = 86400 * $period;
    }
    my $terms; my $url_terms;
    if ($blog_id) {
        $terms->{blog_id} = $blog_id;
        $url_terms->{blog_id} = $blog_id;
    } else {
        $blog = MT::Blog->load( undef, { limit => 1 } );
    }
    my $newest_log = LogViewer4MTOS::LogViewer->load( $terms, { sort => 'timestamp', direction => 'descend', limit => 1 } );
    unless (defined $newest_log) {
        return $app->trans_error('Error: Log record was not found.');
    }
    $url_terms->{period} = 'all';
    if ($object_type) {
        $url_terms->{object_type} = $object_type;
    }
    my @urls = LogViewer4MTOS::LogViewerUrls->load($url_terms);
    my $days_to = $newest_log->timestamp;
    my $year_key = substr($days_to, 0, 4);
    my $month_key = substr($days_to, 0, 6);
    my $ut = &_get_unixtime($days_to);
    my $days_from = $ut - $period_sec;
    my @tl = &offset_time_list($days_from, $blog);
    $days_from = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
    my $res; my @items;  
    my $max = 0;
    for my $url_record (@urls) {
        my $log_terms;
        if ($blog_id) {
            $log_terms->{blog_id} = $blog_id;
        }
        my $url = $url_record->url;
        my $title = $url_record->title;
        my $last_accsess = $url_record->last_access;
        my $object_type = $url_record->object_type;
        $title =~ s/\\'/'/g;
        $title =~ s/&amp;/&/g;
        $log_terms->{url} = $url;
        my $count_log = LogViewer4MTOS::LogViewer->count( $log_terms, {
                                                    sort => 'timestamp',
                                                    start_val => $days_from,
                                                    direction => 'ascend',
                                                    } );

        if ($count_log) {
            if ($max < $count_log) {
                $max = $count_log;
            }
            my $scount = sprintf "%07d" , $count_log;
            push (@items, "$scount\t$title\t$url\t$count_log\t$last_accsess\t$object_type");
            my $object_type = $url_record->object_type;
            $total = $total + $count_log;
            if ($object_type eq 'entry') {
                $entry_total = $entry_total + $count_log;
            } elsif ($object_type eq 'page') {
                $page_total = $page_total + $count_log;
            } elsif ($object_type eq 'daily') {
                $daily_total = $daily_total + $count_log;
            } elsif ($object_type eq 'category') {
                $category_total = $category_total + $count_log;
            } elsif ($object_type eq 'index') {
                $index_total = $index_total + $count_log;
            } else {
                $other_total = $other_total + $count_log;
            }            
        }
    }
    my @year_list; my @month_list; my @day_list; my $bid;
    unless ($blog_id) {
        $bid = 0;
    } else {
        $bid = $blog_id;
    }
        @year_list  = LogViewer4MTOS::LogSummary->load( { 
                                                     blog_id => $bid,
                                                     period     => 'yearly', },
                                                   { sort       => 'period_key',
                                                     direction  => 'descend'} );
        @month_list = LogViewer4MTOS::LogSummary->load( { 
                                                     blog_id => $bid,
                                                     period_year => $year_key,
                                                     period     => 'monthly', },
                                                   { sort       => 'period_key',
                                                     direction  => 'descend'} );
        @day_list = LogViewer4MTOS::LogSummary->load( { 
                                                   blog_id      => $bid,
                                                   period_month => $month_key,
                                                   period       => 'daily', },
                                                 { sort         => 'period_key',
                                                   direction    => 'descend'} );
    my @opt_year_loop;
    for my $opt_year (@year_list) {
        my $ts = $opt_year->period_key;
        my $current_year = 0;
        $ts .= '010101000000';
        push @opt_year_loop,
            {
                year_key     => $opt_year->period_key,
                year_label   => &format_ts('%Y', $ts, undef),
                current_year => $current_year,
            };
    }
    
    my @opt_month_loop; 
    for my $opt_month (@month_list) {
        my $ts = $opt_month->period_key;
        my $current_month = 0;
        $ts .= '01000000';
        push @opt_month_loop,
            {
                month_key     => $opt_month->period_key,
                month_label   => &format_ts('%Y/%m', $ts, undef),
                current_month => $current_month,
            };
    }
    
    my @opt_day_loop; 
    for my $opt_day (@day_list) {
        my $ts = $opt_day->period_key;
        my $current_day = 0;
        $ts .= '000000';
        push @opt_day_loop,
            {
                day_key     => $opt_day->period_key,
                day_label   => &format_ts('%Y/%m/%d', $ts, undef),
                current_day => $current_day,
            };
    }
    my $count_user;
    if ($blog_id) {
        $count_user = LogViewer4MTOS::LogViewerUserInfo->count( undef
                                                , {
                                                'join' => [ 'LogViewer4MTOS::LogViewer', 'userinfo_id',
                                                { blog_id => $blog_id },
                                                { 'sort' => 'timestamp',
                                                  start_val => $days_from,
                                                  direction => 'ascend',
                                                  unique => 1,
                                                } ]
                                                });
    } else {
        $count_user = LogViewer4MTOS::LogViewerUserInfo->count( undef
                                                , {
                                                'join' => [ 'LogViewer4MTOS::LogViewer', 'userinfo_id',
                                                 undef,
                                                { 'sort' => 'timestamp',
                                                  start_val => $days_from,
                                                  direction => 'ascend',
                                                  unique => 1,
                                                } ]
                                                });
    }

    my %param;
    my @tmpl_loop; my $odd = 1;
    $param{'rank_label'} = $plugin->translate('Rank');
    $param{'table_header'} = $plugin->translate('Title');
    $param{'counter_label'} = $plugin->translate('Access');
    $param{'type_label'} = $plugin->translate('Type');
    $param{'last_access_label'} = $plugin->translate('Last Access');
    $param{'no_select_label'} = $plugin->translate('Select...');
    $param{'meta_total'} = $total;
    $param{'meta_entry_total'} = $entry_total;
    $param{'meta_page_total'} = $page_total;
    $param{'meta_category_total'} = $category_total;
    $param{'meta_daily_total'} = $daily_total;
    $param{'meta_index_total'} = $index_total;
    $param{'meta_other_total'} = $other_total;
    $param{'meta_cout_user'} = $count_user;
    $param{'all_label'} = $plugin->translate('All');
    $param{'last7days_label'} = $plugin->translate('Last 7 days');
    $param{'last1days_label'} = $plugin->translate('Last 24 hours');
    $param{'label_total'} = $plugin->translate('Total');
    $param{'label_last'} = $plugin->translate('Last');
    $param{'label_days'} = $plugin->translate('Days');
    $param{'view_last'} = $period;
    $param{'view_access_log'} = 1;
    my $i = $offset;
    $max = 60 / $max;
    @items = sort {$b <=> $a} @items;
    
    for my $item (@items) {
        $i ++;
        my @metas = split(/\t/, $item);
        my $item_title = $metas[1];
        my $item_url = $metas[2];
        my $item_count = $metas[3];
        my $item_last_access = $metas[4];
        my $item_object_type = $metas[5];
        push @tmpl_loop,
            {
                page_rank    => $i,
                page_url     => $item_url,
                page_name    => $item_title,
                page_type    => $item_object_type,
                page_counter => $item_count,
                page_percent => (int (($item_count / $total) * 1000)) / 10,
                page_bar_length => int($max * $item_count) + 1,
                last_access  => &format_ts(undef, $item_last_access, undef),
                odd => $odd,
            };
        if ($odd == 1) { $odd = 0 } else { $odd = 1 };
    }

    my $rank_start = $offset + 1;
    my $period_label = $plugin->translate('Last [_1] days', $period);
    my $page_title = $plugin->translate('URL Report') . ' ( ' . $period_label . ' : ' . $plugin->translate('Rank');
    if ($i) {
        $page_title .= ' ' . $rank_start . ' - ' . $i;
    }
    $page_title .= ' )';
    $param{'page_title'} = $page_title;
    $app->{plugin_template_path} = File::Spec->catdir($plugin->path,'tmpl');
    $param{'tmpl_loop'} = \@tmpl_loop;
    $param{'opt_year_loop'} = \@opt_year_loop;
    $param{'opt_month_loop'} = \@opt_month_loop;
    $param{'opt_day_loop'} = \@opt_day_loop;

    my $tmpl = 'log_view.tmpl';
    return $app->build_page($tmpl, \%param);
}

sub _make_report {
    my $app = shift;
    my $q = $app->param;
    my $key = $q->param('key');
    my $task = $q->param('task');
    $app->{plugin_template_path} = File::Spec->catdir($plugin->path,'tmpl');
    if ($key eq 'finish') {

        my %param;
        $param{'page_title'} = $plugin->translate('Import Log');
        $param{'page_msg'} = $plugin->translate('Import access log &amp all report has been rebuild.');
        $param{'view_report'} = $plugin->translate('View Report');
        
        my $tmpl = 'import_log.tmpl';
        return $app->build_page($tmpl, \%param);
    }
    
    if ($key eq 'lastdays') {
        my $newest_day;
        my $blog = MT::Blog->load( undef, { limit => 1 } );
        my @keys = ('last7days', 'last1days');
        for my $period_key (@keys) {
            my $period_sec;
            if ($period_key eq 'last7days') {
                $period_sec = 86400 * 7;
            } else {
                $period_sec = 86400;
            }
            my $newest_log = LogViewer4MTOS::LogViewer->load( undef, { sort => 'timestamp', direction => 'descend', limit => 1 } );
            unless (defined $newest_log) {
                return $app->trans_error('Error: Log record was not found.');
            }
            my @urls = LogViewer4MTOS::LogViewerUrls->load( { period => 'all' } );
            my $days_to = $newest_log->timestamp;
            $newest_day = $days_to;
            my $ut = &_get_unixtime($days_to);
            my $days_from = $ut - $period_sec;
            my @tl = &offset_time_list($days_from, $blog);
            $days_from = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
            my $total = 0; my $entry_total = 0; my $page_total = 0; my $category_total = 0;
            my $daily_total = 0; my $index_total = 0; my $other_total = 0;
            for my $url_record (@urls) {
                my $url = $url_record->url;
                my $title = $url_record->title;
                my $object_type = $url_record->object_type;
                my $object_id = $url_record->object_id;
                my $url_blog_id = $url_record->blog_id;
                my $first_access = $url_record->first_access;
                my $last_access = $url_record->last_access;
                my $count_log = LogViewer4MTOS::LogViewer->count( {
                                                             url => $url,
                                                            }
                                                            , {
                                                             sort => 'timestamp',
                                                             start_val => $days_from,
                                                             direction => 'ascend',
                                                            } );
                if ($count_log) {
                    my $last_url_record = LogViewer4MTOS::LogViewerLastUrls->load( { url => $url, period => $period_key } );
                    unless (defined $last_url_record) {
                        $last_url_record = LogViewer4MTOS::LogViewerLastUrls->new;
                        $last_url_record->url($url);
                    }
                    $last_url_record->blog_id($url_blog_id);
                    $last_url_record->title($title);
                    $last_url_record->modifier_ts($days_to);
                    $last_url_record->object_id($object_id);
                    $last_url_record->object_type($object_type);
                    if ( ($object_type eq 'entry') || ($object_type eq 'page') ) {
                        $last_url_record->entry_id($object_id);
                    }
                    $last_url_record->first_access($first_access);
                    $last_url_record->last_access($last_access);
                    $last_url_record->counter($count_log);
                    $last_url_record->period($period_key);
                    $last_url_record->save
                        or return $app->trans_error('Error saving log record last urls: [_1]', $last_url_record->errstr);
                    $total = $total + $count_log;
                    if ($object_type eq 'entry') {
                        $entry_total = $entry_total + $count_log;
                    } elsif ($object_type eq 'page') {
                        $page_total = $page_total + $count_log;
                    } elsif ($object_type eq 'daily') {
                        $daily_total = $daily_total + $count_log;
                    } elsif ($object_type eq 'category') {
                        $category_total = $category_total + $count_log;
                    } elsif ($object_type eq 'index') {
                        $index_total = $index_total + $count_log;
                    } else {
                        $other_total = $other_total + $count_log;
                    }
                }
            }

            # uniq user
            my $count_user = LogViewer4MTOS::LogViewerUserInfo->count( undef
                                                        , {
                                                        'join' => [ 'LogViewer4MTOS::LogViewer', 'userinfo_id',
                                                        undef,
                                                        { 'sort' => 'timestamp',
                                                          start_val => $days_from,
                                                          direction => 'ascend',
                                                          unique => 1,
                                                        } ]
                                                        });
                                                        
            my $last_summary = LogViewer4MTOS::LogSummary->load( { blog_id => 0, period => $period_key, } );
            unless (defined $last_summary) {
                $last_summary = LogViewer4MTOS::LogSummary->new;
                $last_summary->blog_id(0);
                $last_summary->period($period_key);
            }
            $last_summary->counter_user($count_user);
            $last_summary->counter($total);
            $last_summary->counter_entry($entry_total);
            $last_summary->counter_page($page_total);
            $last_summary->counter_category($category_total);
            $last_summary->counter_daily($daily_total);
            $last_summary->counter_index($index_total);
            $last_summary->counter_other($other_total);
            $last_summary->save 
                    or return $app->trans_error('Error saving log record last summary: [_1]', $last_summary->errstr);
            my @blogs = MT::Blog->load();
            my @blog_ids;
            for my $tmp_blog (@blogs) {
                push (@blog_ids, $tmp_blog->id);
            }
            my @abject_types = ('entry', 'page', 'category', 'daily', 'index', 'other');
            for my $blog_id (@blog_ids) { 
                my $total = 0; my $entry_total = 0; my $page_total = 0; my $category_total = 0;
                my $daily_total = 0; my $index_total = 0; my $other_total = 0;
                $total = LogViewer4MTOS::LogViewer->count( {
                                                       blog_id => $blog_id,
                                                      }
                                                      , {
                                                       sort => 'timestamp',
                                                       start_val => $days_from,
                                                       direction => 'ascend',
                                                      } );
                for my $object_type (@abject_types) {
                    my $count_log = LogViewer4MTOS::LogViewer->count( {
                                                                 blog_id => $blog_id,
                                                                 object_type => $object_type,
                                                                }
                                                                , {
                                                                 sort => 'timestamp',
                                                                 start_val => $days_from,
                                                                 direction => 'ascend',
                                                                } );
                    if ($object_type eq 'entry') {
                        $entry_total = $count_log;
                    } elsif ($object_type eq 'page') {
                        $page_total = $count_log;
                    } elsif ($object_type eq 'daily') {
                        $daily_total = $count_log;
                    } elsif ($object_type eq 'category') {
                        $category_total = $count_log;
                    } elsif ($object_type eq 'index') {
                        $index_total = $count_log;
                    } else {
                        $other_total = $count_log;
                    }
                }
                my $last_summary = undef;
                my $count_user = undef;
                # uniq user
                $count_user = LogViewer4MTOS::LogViewerUserInfo->count( undef
                                                            , {
                                                            'join' => [ 'LogViewer4MTOS::LogViewer', 'userinfo_id',
                                                            { blog_id => $blog_id },
                                                            { 'sort' => 'timestamp',
                                                              start_val => $days_from,
                                                              direction => 'ascend',
                                                              unique => 1,
                                                            } ]
                                                            });
                $last_summary = LogViewer4MTOS::LogSummary->load( { blog_id => $blog_id, period => $period_key, } );
                unless (defined $last_summary) {
                    $last_summary = LogViewer4MTOS::LogSummary->new;
                    $last_summary->blog_id($blog_id);
                    $last_summary->period($period_key);
                }
                $last_summary->counter_user($count_user);
                $last_summary->counter($total);
                $last_summary->counter_entry($entry_total);
                $last_summary->counter_page($page_total);
                $last_summary->counter_category($category_total);
                $last_summary->counter_daily($daily_total);
                $last_summary->counter_index($index_total);
                $last_summary->counter_other($other_total);
                $last_summary->save 
                        or return $app->trans_error('Error saving log record last summary: [_1]', $last_summary->errstr);
            }
        }
        
        my @remove_logs = LogViewer4MTOS::LogViewerLastUrls->load( undef, 
                                                              {
                                                               sort => 'modifier_ts',
                                                               direction => 'descend',
                                                               start_val => $newest_day,
                                                               } );
        for my $rem_item (@remove_logs) {
            $rem_item->remove
                 or return $app->trans_error('Error clean up log record last summary: [_1]', $rem_item->errstr);
        }
        return $app->redirect($app->uri(mode => 'make_report', args => { 'key' => 'finish' } ));
    }
    
    my $ts = $q->param('ts');
    my $session_key = $q->param('session_key');

    my $sess = LogViewer4MTOS::LogReportSession->load( {
                                                    ts => $ts,
                                                    session_key => $session_key,
                                                    key => $key,
                                                    task => $task,
                                                    
                                                   }, { limit => 1, } );
    unless (defined $sess) {
        return "$ts $session_key $key $task";
        return $app->trans_error('Error loading log report session.');
    }
#    MT::Util::start_background_task(sub { make_log_report($ts, $session_key) });
    
    my @log_records = LogViewer4MTOS::LogViewer->load( { import_time => $ts, session_key => $session_key },
                                                  { sort => 'url' } );
    my @modifier_urls;
    
    for my $log_record (@log_records) {
        my $timestamp = $log_record->timestamp;
        my $blog_id = $log_record->blog_id;
        my $object_type = $log_record->object_type;
        my $object_id = $log_record->object_id;
        my $url = $log_record->url;
        my $title = $log_record->title;
        my $search_str = $log_record->search_str;
        my $referrer = $log_record->referrer;
        my $referrer_self = $log_record->referrer_self;
        my $user_name = $log_record->user_name;
        my $user_id = $log_record->user_id;
        my $userinfo = $log_record->userinfo;
        my $yy = substr($timestamp, 0, 4);
        my $yymm = substr($timestamp, 0, 6);
        my $yymmdd = substr($timestamp, 0, 8);

        my $comp; my $u_check_start;  my $u_check_end;
        if ($key eq 'all') {
            $comp = $url;
            $u_check_start = '19000101000000';
            $u_check_end = '21000000000000';
        } elsif ($key eq 'yearly') {
            $comp = $url . "\t" . $yy;
            $u_check_start = $yy . '0101000000';
            $u_check_end = $yy . '1231235959';
        } elsif ($key eq 'monthly') {
            $comp = $url . "\t" . $yymm;
            $u_check_start = $yymm . '01000000';
            my @start_end_month = &start_end_month($u_check_start);
            $u_check_end = $start_end_month[1];
        } elsif ($key eq 'daily') {
            $comp = $url . "\t" . $yymmdd;
            $u_check_start = $yymmdd . '000000';
            $u_check_end = $yymmdd . '235959';
        }
        if ( ($key eq 'all') || ($key eq 'yearly') || ($key eq 'monthly') || ($key eq 'daily') ) {
        # all
            my $terms;
            $terms->{'import_time'} = $ts;
            $terms->{'session_key'} = $session_key;
            $terms->{'url'} = $url;
            if ($key eq 'yearly') {
                $terms->{'ts_year'} = $yy;
            } elsif ($key eq 'monthly') {
                $terms->{'ts_month'} = $yymm;
            } elsif ($key eq 'daily') {
                $terms->{'ts_day'} = $yymmdd;
            }
            if ($task eq 'url') {
                unless ( grep(/^$comp$/, @modifier_urls) ) {
                    push (@modifier_urls, $comp);

                    my $log_count = LogViewer4MTOS::LogViewer->count( $terms );
                    my $last_record = LogViewer4MTOS::LogViewer->load(  $terms, {
                                                                   sort => 'timestamp',
                                                                   direction => 'descend',
                                                                   limit => 1,
                                                                   } ),
                    my $log_viewer_urls;
                    if ($key eq 'all') {
                        $log_viewer_urls = LogViewer4MTOS::LogViewerUrls->load( { url => $url, period => 'all' } );
                    } elsif ($key eq 'yearly') {
                        $log_viewer_urls = LogViewer4MTOS::LogViewerUrls->load( { url => $url, period => 'yearly', period_key => $yy, } );
                    } elsif ($key eq 'monthly') {
                        $log_viewer_urls = LogViewer4MTOS::LogViewerUrls->load( { url => $url, period => 'monthly', period_key => $yymm, } );
                    } elsif ($key eq 'daily') {
                        $log_viewer_urls = LogViewer4MTOS::LogViewerUrls->load( { url => $url, period => 'daily', period_key => $yymmdd, } );
                    }
                    unless (defined $log_viewer_urls) {
                        my $first_record = LogViewer4MTOS::LogViewer->load( $terms, {
                                                                        sort => 'timestamp',
                                                                        direction => 'descend',
                                                                        limit => 1,
                                                                        } ),
                        $log_viewer_urls = LogViewer4MTOS::LogViewerUrls->new;
                        $log_viewer_urls->url($url);
                        $log_viewer_urls->counter(0);
                        $log_viewer_urls->first_access($first_record->timestamp);
                        $log_viewer_urls->period($key);
                        if ($key eq 'yearly') {
                            $log_viewer_urls->period_key($yy);
                        } elsif ($key eq 'monthly') {
                            $log_viewer_urls->period_key($yymm);
                        } elsif ($key eq 'daily') {
                            $log_viewer_urls->period_key($yymmdd);
                        }
                    }
                    my $counter = $log_viewer_urls->counter;
                    $log_viewer_urls->counter($counter + $log_count);
                    $log_viewer_urls->title($title);
                    $log_viewer_urls->last_access($last_record->timestamp);
                    $log_viewer_urls->blog_id($blog_id);
                    $log_viewer_urls->object_type($object_type);
                    $log_viewer_urls->object_id($object_id);
                    if ( ($object_type eq 'entry') || ($object_type eq 'page') ) {
                        $log_viewer_urls->entry_id($object_id);
                    }
                    $log_viewer_urls->save
                            or return $app->trans_error('Error saving log record urls: [_1]', $log_viewer_urls->errstr);
                }
            } elsif ($task eq 'summary') {

                unless ( grep(/^$comp$/, @modifier_urls) ) {
                    push (@modifier_urls, $comp);
                    my $log_count = LogViewer4MTOS::LogViewer->count( $terms );
                    my $log_viewer_summary_all; my $log_viewer_summary_blog;
                    my $terms_all;
                    $terms_all->{'period'} = $key;
                    if ($key eq 'yearly') {
                        $terms_all->{'period_key'} = $yy;
                    } elsif ($key eq 'monthly') {
                        $terms_all->{'period_key'} = $yymm;
                    } elsif ($key eq 'daily') {
                        $terms_all->{'period_key'} = $yymmdd;
                    }
                    $terms_all->{'blog_id'} = 0;
                    $log_viewer_summary_all  = LogViewer4MTOS::LogSummary->load( $terms_all );
                    $terms_all->{'blog_id'} = $blog_id;
                    $log_viewer_summary_blog = LogViewer4MTOS::LogSummary->load( $terms_all );
                    unless (defined $log_viewer_summary_all) {
                        $log_viewer_summary_all = LogViewer4MTOS::LogSummary->new;
                        $log_viewer_summary_all->counter(0);
                        $log_viewer_summary_all->blog_id(0);
                        $log_viewer_summary_all->period($key);
                        if ($key eq 'yearly') {
                            $log_viewer_summary_all->period_key($yy);
                            $log_viewer_summary_all->period_year($yy);
                        } elsif ($key eq 'monthly') {
                            $log_viewer_summary_all->period_key($yymm);
                            $log_viewer_summary_all->period_month($yymm);
                            $log_viewer_summary_all->period_year($yy);
                        } elsif ($key eq 'daily') {
                            $log_viewer_summary_all->period_key($yymmdd);
                            $log_viewer_summary_all->period_month($yymm);
                            $log_viewer_summary_all->period_year($yy);
                        }
                    }
                    unless (defined $log_viewer_summary_blog) {
                        $log_viewer_summary_blog = LogViewer4MTOS::LogSummary->new;
                        $log_viewer_summary_blog->counter(0);
                        $log_viewer_summary_blog->blog_id($blog_id);
                        $log_viewer_summary_blog->period($key);
                        if ($key eq 'yearly') {
                            $log_viewer_summary_blog->period_key($yy);
                            $log_viewer_summary_blog->period_year($yy);
                        } elsif ($key eq 'monthly') {
                            $log_viewer_summary_blog->period_key($yymm);
                            $log_viewer_summary_blog->period_month($yymm);
                            $log_viewer_summary_blog->period_year($yy);
                        } elsif ($key eq 'daily') {
                            $log_viewer_summary_blog->period_key($yymmdd);
                            $log_viewer_summary_blog->period_month($yymm);
                            $log_viewer_summary_blog->period_year($yy);
                        }
                    }
                    my $a_s_counter = $log_viewer_summary_all->counter;
                    my $b_s_counter = $log_viewer_summary_blog->counter;
                    $log_viewer_summary_all->counter($a_s_counter + $log_count);
                    $log_viewer_summary_blog->counter($b_s_counter + $log_count);
                    
                    my $count_user = LogViewer4MTOS::LogViewerUserInfo->count( undef
                                                                        , {
                                                                        'join' => [ 'LogViewer4MTOS::LogViewer', 'userinfo_id',
                                                                        { 
                                                                          timestamp => [$u_check_start, $u_check_end], },
                                                                        { range  => { timestamp => 1 },
                                                                          unique => 1,
                                                                        } ]
                                                                        });
                    my $count_blog_user = LogViewer4MTOS::LogViewerUserInfo->count( undef
                                                                        , {
                                                                        'join' => [ 'LogViewer4MTOS::LogViewer', 'userinfo_id',
                                                                        { 
                                                                          blog_id => $blog_id,
                                                                          timestamp => [$u_check_start, $u_check_end], },
                                                                        { range  => { timestamp => 1 },
                                                                          unique => 1,
                                                                        } ]
                                                                        });
                    $log_viewer_summary_all->counter_user($count_user);
                    $log_viewer_summary_blog->counter_user($count_blog_user);
                    
                    if ($object_type eq 'entry') {
                        my $counter_tmp = $log_viewer_summary_all->counter_entry;
                        $log_viewer_summary_all->counter_entry($counter_tmp + $log_count);
                        my $b_counter_tmp = $log_viewer_summary_blog->counter_entry;
                        $log_viewer_summary_blog->counter_entry($b_counter_tmp + $log_count);
                    } elsif ($object_type eq 'page') {
                        my $counter_tmp = $log_viewer_summary_all->counter_page;
                        $log_viewer_summary_all->counter_page($counter_tmp + $log_count);
                        my $b_counter_tmp = $log_viewer_summary_blog->counter_page;
                        $log_viewer_summary_blog->counter_page($b_counter_tmp + $log_count);
                    } elsif ($object_type eq 'category') {
                        my $counter_tmp = $log_viewer_summary_all->counter_category;
                        $log_viewer_summary_all->counter_category($counter_tmp + $log_count);
                        my $b_counter_tmp = $log_viewer_summary_blog->counter_category;
                        $log_viewer_summary_blog->counter_category($b_counter_tmp + $log_count);
                    } elsif ($object_type eq 'daily') {
                        my $counter_tmp = $log_viewer_summary_all->counter_daily;
                        $log_viewer_summary_all->counter_daily($counter_tmp + $log_count);
                        my $b_counter_tmp = $log_viewer_summary_blog->counter_daily;
                        $log_viewer_summary_blog->counter_daily($b_counter_tmp + $log_count);
                    } elsif ($object_type eq 'index') {
                        my $counter_tmp = $log_viewer_summary_all->counter_index;
                        $log_viewer_summary_all->counter_index($counter_tmp + $log_count);
                        my $b_counter_tmp = $log_viewer_summary_blog->counter_index;
                        $log_viewer_summary_blog->counter_index($b_counter_tmp + $log_count);
                    } else {
                        my $counter_tmp = $log_viewer_summary_all->counter_other;
                        $log_viewer_summary_all->counter_other($counter_tmp + $log_count);
                        my $b_counter_tmp = $log_viewer_summary_blog->counter_other;
                        $log_viewer_summary_blog->counter_other($b_counter_tmp + $log_count);
                    }
                    $log_viewer_summary_all->save 
                            or return $app->trans_error('Error saving log summary: [_1]', $log_viewer_summary_all->errstr);

                    $log_viewer_summary_blog->save 
                            or return $app->trans_error('Error saving log summary: [_1]', $log_viewer_summary_blog->errstr);
                }
            }
        }
    }
    $sess->remove
            or return $app->trans_error('Error removing log report session: [_1]', $sess->errstr);
    my $next_sess = LogViewer4MTOS::LogReportSession->load( {
                                                    ts => $ts,
                                                    session_key => $session_key,
                                                   });
    if (defined $next_sess) {
        #return $app->redirect($app->uri(mode => 'make_report', args => { 'ts' => $ts, 'session_key' => $session_key,
        #                                                                 'key' => $next_sess->key, 'task' => $next_sess->task } ));
        #$app->{plugin_template_path} = File::Spec->catdir($plugin->path,'tmpl');
        my $next_sess_key = $next_sess->key;
        my $next_sess_task = $next_sess->task;
        my %param; my $page_msg;
        $param{'page_title'} = $plugin->translate('Rebuilding Report');
        if ($next_sess_key eq 'all') {
            if ($next_sess_task eq 'url') {
                $page_msg = $plugin->translate('Rebuilding all url report...');
            } elsif ($next_sess_task eq 'summary') {
                $page_msg = $plugin->translate('Rebuilding all summary report...');
            }
        } elsif ($next_sess_key eq 'yearly') {
            if ($next_sess_task eq 'url') {
                $page_msg = $plugin->translate('Rebuilding yearly url report...');
            } elsif ($next_sess_task eq 'summary') {
                $page_msg = $plugin->translate('Rebuilding yearly summary report...');
            }
        } elsif ($next_sess_key eq 'monthly') {
            if ($next_sess_task eq 'url') {
                $page_msg = $plugin->translate('Rebuilding monthly url report...');
            } elsif ($next_sess_task eq 'summary') {
                $page_msg = $plugin->translate('Rebuilding monthly summary report...');
            }
        } elsif ($next_sess_key eq 'daily') {
            if ($next_sess_task eq 'url') {
                $page_msg = $plugin->translate('Rebuilding daily url report...');
            } elsif ($next_sess_task eq 'summary') {
                $page_msg = $plugin->translate('Rebuilding daily summary report...');
            }
        }
        $param{'rebuild_report'} = 1;
        $param{'page_msg'} = $page_msg;
        my $base = $app->base;
        my $return_url = $app->uri(mode => 'make_report', args => { 'ts' => $ts, 'session_key' => $session_key,
                                                                         'key' => $next_sess_key, 'task' => $next_sess->task } );
        my $js = <<"JS";
var counter = 0;
var eid;
function func() {
    if (counter != 0) {
        clearTimeout(eid);
        counter = 0;
    } else {
        counter = 1;
        eid = setTimeout("func(window.location.href = '$base$return_url')", 200);
    }
}
func();
JS
        $param{'location_redirect'} = $js;
        my $tmpl = 'import_log.tmpl';
        return $app->build_page($tmpl, \%param);
    } else {
        unless ($key eq 'lastdays') {
            my %param;
            $param{'page_title'} = $plugin->translate('Rebuilding Report');
            $param{'rebuild_report'} = 1;
            $param{'page_msg'} = $plugin->translate('Finishing report...');
            my $base = $app->base;
            my $return_url = $app->uri(mode => 'make_report', args => { 'key' => 'lastdays' } );
            my $js = <<"JS";
var counter = 0;
var eid;
function func() {
    if (counter != 0) {
        clearTimeout(eid);
        counter = 0;
    } else {
        counter = 1;
        eid = setTimeout("func(window.location.href = '$base$return_url')", 200);
    }
}
func();
JS
            $param{'location_redirect'} = $js;
            my $tmpl = 'import_log.tmpl';
            return $app->build_page($tmpl, \%param);
        } else {
            return $app->redirect($app->uri(mode => 'make_report', args => { 'key' => 'finish' } ));
        }
    }
}

sub _tracking_log {

}

sub _get_unixtime {
    my $date = shift;
    my $myear = substr($date, 0, 4); my $mmonth = substr($date, 4, 2);
    my $mday = substr($date, 6, 2); my $mhour = substr($date, 8, 2);
    my $mminutes = substr($date, 10, 2); my $mseconds = substr($date, 12, 2); my $dayspast = 0;
    for (my $i = 1970; $i < $myear; $i++) {
        my $yeardays = ($i % 4 == 0 && ($i % 400 == 0 || $i % 100 != 0)) ? 366 : 365;
        $dayspast = $dayspast + $yeardays;
    }
    my @days_in_month = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
    for (my $i = 1; $i < $mmonth; $i++) {
        my $monthdays = ($i == 2 && $myear % 4 == 0 && ($myear % 400 == 0 || $myear % 100 != 0)) ? 29 : $days_in_month[$i-1];
        $dayspast = $dayspast + $monthdays;
    }
    $dayspast = $dayspast + $mday - 1;
    $dayspast = $dayspast * 86400;
    $dayspast = $dayspast + $mhour * 3600;
    $dayspast = $dayspast + $mminutes * 60;
    return $dayspast + $mseconds;
}

sub _int2wday {
    my $int = shift;
    if ($int == 0) {
        return $plugin->translate('Sunday');
    } elsif ($int == 1) {
        return $plugin->translate('Monday');
    } elsif ($int == 2) {
        return $plugin->translate('Tuesday');
    } elsif ($int == 3) {
        return $plugin->translate('Wednesday');
    } elsif ($int == 4) {
        return $plugin->translate('Thursday');
    } elsif ($int == 5) {
        return $plugin->translate('Friday');
    } elsif ($int == 6) {
        return $plugin->translate('Saturday');
    }
}

1;